"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.promise");

require("core-js/modules/es6.object.to-string");

var _react = _interopRequireDefault(require("react"));

var _utils = require("./utils");

var _collect = _interopRequireDefault(require("./collect"));

var _vuelike = _interopRequireDefault(require("./vuelike"));

var _async = _interopRequireDefault(require("./async"));

var _constructor = require("./constructor");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _createElement = _react.default.createElement;
var _cloneElement = _react.default.cloneElement;
var _forceUpdate = _react.default.Component.prototype.forceUpdate;
var builtInTags = {
  template: _react.default.Fragment,
  async: _async.default,
  observer: _utils.Observer
};

var createElement = function createElement(Component, props) {
  for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  if (_collect.default.elements) return _collect.default.push(_createElement, Component, props, children);
  return _createElement.call.apply(_createElement, [this, Component, props].concat(children));
};

function createElementHook(Component, props) {
  var _this = this;

  for (var _len2 = arguments.length, children = new Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
    children[_key2 - 2] = arguments[_key2];
  }

  // const $component = props && props.$component;
  // if ($component) {
  //   Component = $component;
  //   delete props.$component;
  // }
  if (!Component) return _createElement.call.apply(_createElement, [this, Component, props].concat(children));
  children.forEach(function (c, i) {
    if (c instanceof Promise) children[i] = createElement.call(_this, _async.default, {
      promise: c
    });
  });
  var builtIn;

  if (typeof Component === 'string') {
    var _builtIn = builtInTags[Component];
    if (_builtIn !== undefined) Component = _builtIn;
  }

  var isVuelike = false;

  if (Component.vuelikeConstructor) {
    isVuelike = true;
    var newComponent = Component.vuelikeConstructor(Component, props || {}, children, _react.default);
    if (newComponent !== undefined) Component = newComponent;
  }

  if (Component.beforeConstructor) {
    var _newComponent = Component.beforeConstructor(Component, props || {}, children, _react.default);

    if (_newComponent !== undefined) Component = _newComponent;
  }

  if (!builtIn && !isVuelike) (0, _constructor.otherConstructor)(props, children, _react.default);
  (0, _constructor.fallbackConstructor)(props, children, _react.default);
  return createElement.call.apply(createElement, [this, Component, props].concat(children));
}

function cloneElementHook(element, props) {
  for (var _len3 = arguments.length, children = new Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
    children[_key3 - 2] = arguments[_key3];
  }

  if (_collect.default.elements) return _collect.default.push(_cloneElement, element, props, children);
  return _cloneElement.call.apply(_cloneElement, [this, element, props].concat(children));
}

function forceUpdateHook() {
  if (this && this._isVueLike && this._checkActive) {
    return this._checkActive(_forceUpdate, arguments);
  }

  return _forceUpdate.apply(this, arguments);
}

(0, _utils.innumerable)(forceUpdateHook, '_isVueLike', true);

function ReactHook() {
  if (_react.default._vueLike) {
    if (_react.default._vueLike.build && _react.default._vueLike.build >= _vuelike.default.build) return;
  }

  _react.default.createElement = createElementHook;
  _react.default.cloneElement = cloneElementHook;

  if (!_react.default.Component.prototype.forceUpdate._isVueLike) {
    _react.default.Component.prototype.forceUpdate = forceUpdateHook;
  }

  (0, _utils.innumerable)(_react.default, '_vueLike', _vuelike.default);
}

ReactHook.createElement = createElementHook;
ReactHook.cloneElement = cloneElementHook;
var _default = ReactHook;
exports.default = _default;